@@ -290,11 +290,22 @@ class CorrectionData(NamedTuple):
     id: str
 
 
+class ReplyData(NamedTuple):
+    to: str
+    id: str
+    fallback_start: Optional[int]
+    fallback_end: Optional[int]
+
+
 class ModerationData(NamedTuple):
     stanza_id: str
-    moderator_jid: str
-    reason: Optional[str] = None
-    timestamp: Optional[str] = None
+    moderator_jid: Optional[str]
+    by: Optional[JID]
+    reason: Optional[str]
+    timestamp: Optional[str]
+    stamp: float
+    is_tombstone: bool
+    occupant_id: Optional[str]
 
 
 class DiscoItems(NamedTuple):
@@ -725,6 +736,8 @@ class OMEMOBundle(NamedTuple):
     spk_signature: bytes
     ik: bytes
     otpks: list[dict[str, str]]
+    device_id: int = -1
+    namespace: str = Namespace.OMEMO_TEMP
 
     def pick_prekey(self) -> dict[str, str]:
         return random.SystemRandom().choice(self.otpks)
@@ -760,10 +773,18 @@ class CommonError:
         self.condition_data = self._error_node.getTagData(self.condition)
         self.app_condition = stanza.getAppError()
         self.type = stanza.getErrorType()
+        self.by = None
         self.jid = stanza.getFrom()
         self.id = stanza.getID()
         self._text = {}
 
+        by = self._error_node.getAttr('by')
+        if by is not None:
+            try:
+                self.by = JID.from_string(by)
+            except Exception:
+                pass
+
         text_elements = self._error_node.getTags('text', namespace=Namespace.STANZAS)
         for element in text_elements:
             lang = element.getXmlLang()
@@ -908,7 +929,7 @@ class MAMData(NamedTuple):
     query_id: str
     archive: JID
     namespace: str
-    timestamp: datetime
+    timestamp: float
 
     @property
     def is_ver_1(self) -> bool:
@@ -955,9 +976,11 @@ class MessageProperties:
     type: MessageType = MessageType.NORMAL
     id: Optional[str] = None
     stanza_ids: list[StanzaIDData] = field(default_factory=list)
+    origin_id: Optional[str] = None
     from_: Optional[JID] = None
     to: Optional[JID] = None
     jid: Optional[JID] = None
+    remote_jid: Optional[JID] = None
     subject = None
     body: Optional[str] = None
     thread: Optional[str] = None
@@ -993,6 +1016,7 @@ class MessageProperties:
     receipt: Optional[ReceiptData] = None
     oob: Optional[OOBData] = None
     correction: Optional[CorrectionData] = None
+    reply_data: Optional[ReplyData] = None
     moderation: Optional[ModerationData] = None
     attention: bool = False
     forms = None
